//FMP_VC
//FMP_VC  EverLost Benchmarks
//FMP_VC  Copyright (C) 2006  Flavio M. de Paula
//FMP_VC
//FMP_VC  This is a modified version of the original one from 
//FMP_VC  opencores.org. 
//FMP_VC  This file is licensed under the GNU Lesser  General
//FMP_VC  Public License. See lgpl.txt for more details.
//FMP_VC
//FMP_VC

/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Packet Assembler                                           ////
////  Assembles Token and Data USB packets                       ////
////                                                             ////
////  Author: Rudolf Usselmann                                   ////
////          rudi@asics.ws                                      ////
////                                                             ////
////                                                             ////
////  Downloaded from: http://www.opencores.org/cores/usb/       ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
////                                                             ////
//// Copyright (C) 2000-2003 Rudolf Usselmann                    ////
////                         www.asics.ws                        ////
////                         rudi@asics.ws                       ////
////                                                             ////
//// This source file may be used and distributed without        ////
//// restriction provided that this copyright statement is not   ////
//// removed from the file and that any derivative work contains ////
//// the original copyright notice and the associated disclaimer.////
////                                                             ////
////     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ////
//// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ////
//// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ////
//// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ////
//// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ////
//// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ////
//// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ////
//// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ////
//// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ////
//// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ////
//// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ////
//// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ////
//// POSSIBILITY OF SUCH DAMAGE.                                 ////
////                                                             ////
/////////////////////////////////////////////////////////////////////

//  CVS Log
//
//  $Id: usbf_pa.v,v 1.6 2003/10/17 02:36:57 rudi Exp $
//
//  $Date: 2003/10/17 02:36:57 $
//  $Revision: 1.6 $
//  $Author: rudi $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: usbf_pa.v,v $
//               Revision 1.6  2003/10/17 02:36:57  rudi
//               - Disabling bit stuffing and NRZI encoding during speed negotiation
//               - Now the core can send zero size packets
//               - Fixed register addresses for some of the higher endpoints
//                 (conversion between decimal/hex was wrong)
//               - The core now does properly evaluate the function address to
//                 determine if the packet was intended for it.
//               - Various other minor bugs and typos
//
//               Revision 1.5  2001/11/04 12:22:45  rudi
//
//               - Fixed previous fix (brocke something else ...)
//               - Majore Synthesis cleanup
//
//               Revision 1.4  2001/09/24 01:15:28  rudi
//
//               Changed reset to be active high async.
//
//               Revision 1.3  2001/09/19 14:38:57  rudi
//
//               Fixed TxValid handling bug.
//
//               Revision 1.2  2001/08/10 08:48:33  rudi
//
//               - Changed IO names to be more clear.
//               - Uniquifyed define names to be core specific.
//
//               Revision 1.1  2001/08/03 05:30:09  rudi
//
//
//               1) Reorganized directory structure
//
//               Revision 1.0  2001/03/07 09:17:12  rudi
//
//
//               Changed all revisions to revision 1.0. This is because OpenCores CVS
//               interface could not handle the original '0.1' revision ....
//
//               Revision 0.1.0.1  2001/02/28 08:10:54  rudi
//               Initial Release
//
//

`include "usbf_defines.v"

module usbf_pa(/*AUTOARG*/
   // Outputs
   tx_data7, tx_data6, tx_data5, tx_data4, tx_data3, tx_data2,
   tx_data1, tx_data0, tx_valid, tx_valid_last, tx_first, rd_next,
	       tx_first_r, //FMP_VC_0
   // Inputs
   clk, rst, tx_ready, send_token, token_pid_sel, send_data,
   data_pid_sel, send_zero_length, tx_data_st
   );

   output tx_first_r; //FMP_VC_0
   
   input  clk, rst;
   
   // UTMI TX Interface
   //FMP_VC_1 //output	[7:0]	tx_data;
   output tx_data7;
   output tx_data6;
   output tx_data5;
   output tx_data4;
   output tx_data3;
   output tx_data2;
   output tx_data1;
   output tx_data0;
   
   output tx_valid;
   output tx_valid_last;
   input  tx_ready;
   output tx_first;
   
   // Protocol Engine Interface
   input  send_token;
   input [1:0] token_pid_sel;
   input       send_data;
   input [1:0] data_pid_sel;
   input       send_zero_length;
   
   // IDMA Interface
   input [7:0] tx_data_st;
   output      rd_next;
   
///////////////////////////////////////////////////////////////////
//
// Local Wires and Registers
//

/* -----\/----- EXCLUDED -----\/-----
//FMP_VC_2
 parameter	[4:0]	// synopsys enum state
		IDLE   = 5'b00001,
		DATA   = 5'b00010,
		CRC1   = 5'b00100,
		CRC2   = 5'b01000,
		WAIT   = 5'b10000;
 -----/\----- EXCLUDED -----/\----- */

   reg [4:0]   /* synopsys enum state */ state, next_state;
   // synopsys state_vector state

   reg 	       last;
   reg 	       rd_next;
   
   reg [7:0]   token_pid, data_pid;	// PIDs from selectors
   reg [7:0]   tx_data_d;
   reg [7:0]   tx_data_data;
   reg 	       dsel;
   reg 	       tx_valid_d;
   reg 	       send_token_r;
   reg [7:0]   tx_spec_data;
   reg 	       crc_sel1, crc_sel2;
   reg 	       tx_first_r;
   reg 	       send_data_r;
   wire        crc16_clr;
   reg [15:0]  crc16;
   //FMP_VC_3//wire	[15:0]	crc16_next;
   //FMP_VC_3//wire	[15:0]	crc16_rev;
   wire        crc16_add;
   reg 	       send_data_r2;
   reg 	       tx_valid_r;
   reg 	       tx_valid_r1;
   reg 	       zero_length_r;
   reg 	       send_zero_length_r;
   
   wire        crc16_next15,crc16_next14,crc16_next13,crc16_next12,crc16_next11,crc16_next10,
	       crc16_next9,crc16_next8,crc16_next7,crc16_next6,crc16_next5,
	       crc16_next4,crc16_next3,crc16_next2,crc16_next1,crc16_next0;
   //FMP_VC_3
   wire [15:0] crc16_next = {crc16_next15,crc16_next14,crc16_next13,crc16_next12,crc16_next11,crc16_next10,
			     crc16_next9,crc16_next8,crc16_next7,crc16_next6,crc16_next5,
			      crc16_next4,crc16_next3,crc16_next2,crc16_next1,crc16_next0};
   //FMP_VC_3
   wire [15:0] crc16_rev = {~crc16[8],~crc16[9],~crc16[10],~crc16[11],~crc16[12],~crc16[13],~crc16[14],
			    ~crc16[15],~crc16[0],~crc16[1],~crc16[2],~crc16[3],~crc16[4],~crc16[5],
			    ~crc16[6],~crc16[7]};
   
   /* FMP_VC_4 */
// synopsys translate_off
`ifdef VCEGAR
   initial begin
      send_zero_length_r      =  1'd0;
      zero_length_r   =  1'd0;
      tx_valid_r1     =  1'd0;
      tx_valid_r      =  1'd0;
      send_token_r    =  1'd0;
      tx_first_r      =  1'd0;
      send_data_r     =  1'd0;
      send_data_r2  =  1'd0;
      crc16   =  16'd0;
      state   =  5'd1;
end
`endif
   // synopsys translate_on

///////////////////////////////////////////////////////////////////
//
// Misc Logic
//

   always @(posedge clk)
     send_zero_length_r <= send_zero_length;
   
`ifdef USBF_ASYNC_RESET
   always @(posedge clk or negedge rst)
`else
     always @(posedge clk)
`endif
       if(!rst)	zero_length_r <= 1'b0;
       else
	 if(last)	zero_length_r <= 1'b0;
	 else
	   if(crc16_clr)	zero_length_r <= send_zero_length_r;
   
   always @(posedge clk)
     tx_valid_r1 <= tx_valid;
   
   always @(posedge clk)
     tx_valid_r <= tx_valid_r1;
   
`ifdef USBF_ASYNC_RESET
   always @(posedge clk or negedge rst)
`else
     always @(posedge clk)
`endif
       if(!rst)	send_token_r <= 1'b0;
       else
	 if(send_token)	send_token_r <= 1'b1;
	 else
	   if(tx_ready)	send_token_r <= 1'b0;
   
   // PID Select
   always @(token_pid_sel)
     case(token_pid_sel)		// synopsys full_case parallel_case
       2'd0: token_pid = {  ~`USBF_T_PID_ACK,   `USBF_T_PID_ACK};
       2'd1: token_pid = { ~`USBF_T_PID_NACK,  `USBF_T_PID_NACK};
       2'd2: token_pid = {~`USBF_T_PID_STALL, `USBF_T_PID_STALL};
       2'd3: token_pid = { ~`USBF_T_PID_NYET,  `USBF_T_PID_NYET};
// synopsys translate_off
       default: token_pid = 8'hde;//FMP_VC_5
// synopsys translate_on
     endcase
   
   always @(data_pid_sel)
     case(data_pid_sel)		// synopsys full_case parallel_case
       2'd0: data_pid = { ~`USBF_T_PID_DATA0, `USBF_T_PID_DATA0};
       2'd1: data_pid = { ~`USBF_T_PID_DATA1, `USBF_T_PID_DATA1};
       2'd2: data_pid = { ~`USBF_T_PID_DATA2, `USBF_T_PID_DATA2};
       2'd3: data_pid = { ~`USBF_T_PID_MDATA, `USBF_T_PID_MDATA};
// synopsys translate_off
       default: data_pid = 8'hde;//FMP_VC_5
// synopsys translate_on
     endcase

// Data path Muxes

   always @(send_token or send_token_r or token_pid or tx_data_data)
     if(send_token || send_token_r)	tx_data_d = token_pid;
     else				tx_data_d = tx_data_data;
   
   always @(dsel or tx_data_st or tx_spec_data)
     if(dsel)	tx_data_data = tx_spec_data;
     else		tx_data_data = tx_data_st;
   
   always @(crc_sel1 or crc_sel2 or data_pid or crc16_rev)
     if(!crc_sel1 && !crc_sel2)	tx_spec_data = data_pid;
     else
       if(crc_sel1)			tx_spec_data = crc16_rev[15:8];	// CRC 1
       else				tx_spec_data = crc16_rev[7:0];	// CRC 2

   // FMP_VC_3  
   //assign tx_data = tx_data_d;
   assign tx_data7 = tx_data_d[7:7];
   assign tx_data6 = tx_data_d[6:6];
   assign tx_data5 = tx_data_d[5:5];
   assign tx_data4 = tx_data_d[4:4];
   assign tx_data3 = tx_data_d[3:3];
   assign tx_data2 = tx_data_d[2:2];
   assign tx_data1 = tx_data_d[1:1];
   assign tx_data0 = tx_data_d[0:0];

   
// TX Valid assignment
   assign tx_valid_last = send_token | last;
   assign tx_valid = tx_valid_d;

   always @(posedge clk)
     tx_first_r <= send_token | send_data;
   
   assign tx_first = (send_token | send_data) & ! tx_first_r;
   
   // CRC Logic
   always @(posedge clk)
     send_data_r <= send_data;
   
   always @(posedge clk)
     send_data_r2 <= send_data_r;
   
   assign crc16_clr = send_data & !send_data_r;
   
   assign crc16_add =  !zero_length_r & (send_data_r & !send_data_r2) | (rd_next & !crc_sel1); 
   
   always @(posedge clk)
     if(crc16_clr)		crc16 <= 16'hffff;
     else
       if(crc16_add)		crc16 <= crc16_next;
   
   
//FMP_VC_1   
//	.crc_out(	crc16_next		) );
usbf_crc16 u1(
	.crc_in(	crc16		),
	.din(	{tx_data_st[0], tx_data_st[1],
		tx_data_st[2], tx_data_st[3],
		tx_data_st[4], tx_data_st[5],
		tx_data_st[6], tx_data_st[7]}	),
	.crc_out15(	crc16_next15		),
	.crc_out14(	crc16_next14		),
	.crc_out13(	crc16_next13		),
	.crc_out12(	crc16_next12		),
	.crc_out11(	crc16_next11		),
	.crc_out10(	crc16_next10		),
	.crc_out9(	crc16_next9		),
	.crc_out8(	crc16_next8		),
	.crc_out7(	crc16_next7		),
	.crc_out6(	crc16_next6		),
	.crc_out5(	crc16_next5		),
	.crc_out4(	crc16_next4		),
	.crc_out3(	crc16_next3		),
	.crc_out2(	crc16_next2		),
	.crc_out1(	crc16_next1		),
	.crc_out0(	crc16_next0		));

    
///////////////////////////////////////////////////////////////////
//
// Transmit/Encode state machine
//

`ifdef USBF_ASYNC_RESET
   always @(posedge clk or negedge rst)
`else
     always @(posedge clk)
`endif
       if(!rst)	state <= 5'b00001;
       else		state <= next_state;
   
   always @(state or send_data or tx_ready or tx_valid_r or send_zero_length_r)
     begin
	next_state = state;	// Default don't change current state
	tx_valid_d = 1'b0;
	dsel = 1'b0;
	rd_next = 1'b0;
	last = 1'b0;
	crc_sel1 = 1'b0;
	crc_sel2 = 1'b0;
	case(state)		// synopsys full_case parallel_case
	  5'b00001:
	    begin
	       if(send_zero_length_r && send_data)
		 begin
		    tx_valid_d = 1'b1;
		    next_state = 5'b10000;
		    dsel = 1'b1;
		 end
	       else
		 if(send_data)			// Send DATA packet
		   begin
		      tx_valid_d = 1'b1;
		      next_state = 5'b00010;
		      dsel = 1'b1;
		   end
	    end
	  5'b00010:
	    begin
	       if(tx_ready && tx_valid_r)
		 rd_next = 1'b1;
	       
	       tx_valid_d = 1'b1;
	       if(!send_data && tx_ready && tx_valid_r)
		 begin
		    dsel = 1'b1;
		    crc_sel1 = 1'b1;
		    next_state = 5'b00100;
		 end
	    end
	  5'b10000:		// In case of early tx_ready ...
	    begin
	       crc_sel1 = 1'b1;
	       dsel = 1'b1;
	       tx_valid_d = 1'b1;
	       next_state = 5'b00100;
	    end
	  5'b00100:
	    begin
	       dsel = 1'b1;
	       tx_valid_d = 1'b1;
	       if(tx_ready)
		 begin
		    last = 1'b1;
		    crc_sel2 = 1'b1;
		    next_state = 5'b01000;
		 end
	       else
		 begin
		    tx_valid_d = 1'b1;
		    crc_sel1 = 1'b1;
		 end
	       
	    end
	  5'b01000:
	    begin
	       dsel = 1'b1;
	       crc_sel2 = 1'b1;
	       if(tx_ready)
		 begin
		    next_state = 5'b00001;
		 end
	       else
		 begin
		    last = 1'b1;
		 end
	       
	    end // case: 5'b01000
// synopsys translate_off
	  default : next_state = 5'h1;//FMP_VC_5
// synopsys translate_on
	endcase
     end
endmodule // usbf_pa


